function [cv M R2 thresh M0] = icda2(X, M)

%
% Iterated Canonical Discriminant Analysis
%   two groups/populations/classes only
%
% [cv M thresh R2 M0] = icda2(X, M);
%
% If M is empty (or not present) draw ROI, fx:
%
%   [cv M R2 thresh] = icda2(X) or [cv M R2 thresh M0] = icda2(X, []);
%
% Input
%   X - input cube
%   M - mask image, optional
%
% Output
%   cv      - canonical variate
%   M       - final mask
%   R2      - squared canonical correlation over iterations
%   thresh  - optimal threshold (for las iteration)
%   M0      - initial mask
%

% (c) Copyright 2012
% Allan Aasbjerg Nielsen, Ph.D, M.Sc.
% aa@space.dtu.dk, www.imm.dtu.dk/~aa
% 16 May 2012

% foreground and background forced to have same variance?
samevariance = true; % true slower! but more accurate?
if samevariance
    disp('foreground and background have same variances')
else
    disp('foreground and background have different variances')
end

nout = 1; % this is the only valid value of nout for icda2
lambda = 0;
itermax = 20;
epsi = 1e-6;

p = size(X);
nrows = p(1);
ncols = p(2);
if length(p)==2
%     nvars = 1;
    error('icda2: image must be multispectral');
else
    nvars = p(3);
end
N = nrows*ncols;

if nargin < 2 || isempty(M)
    figure
    if nvars > 81; %== 126 || nvars == 110 % 126: HyMap;
        imshowrgb(X, [27 81 16])
    elseif nvars == 6 % TM?
        imshowrgb(X, [5 4 3])
    elseif nvars == 4 % MSS????
        imshowrgb(X, [4 2 1])
    elseif nvars == 3 % SPOT?
        imshowrgb(X, [3 2 1])
    elseif nvars == 2
        imshowrgb(X, [2 2 1])
    else    
        imshowrgb(X,1)
    end
    M = roipoly;
else
    M = logical(M);
    [nrm ncm] = size(M);
    if nrm ~= nrows || ncm ~= ncols
        error('image cube and mask don''t match')
    end
end
M0 = M;

% % norm spectra for each pixel to unit integral (i.e., make pdf-look-alike)
% aux = sum(X, 3);
% X = X./repmat(aux, [1 1 nvars]);

X = reshape(X, N, nvars);
M = reshape(M, N, 1);

Mprev = reshape(M0, N, 1);
R2 = NaN(itermax,1);
for iter=1:itermax
    idx  = find(M);
    idxm = find(~M);
    n  = length(idx);
    nm = length(idxm);
    mfg = mean(X(idx ,:));
    mbg = mean(X(idxm,:));
    mm  = (n*mfg+nm*mbg)/(n+nm); %mean(X);
    % Among
    A  = n*(mfg-mm)'*(mfg-mm) + nm*(mbg-mm)'*(mbg-mm);
    % Total
    T  = (X-repmat(mm,N,1))'*(X-repmat(mm,N,1)); % T = cov(X)*(N-1);
    V = T\(mfg-mbg)';
    % unit vectors
    aux1 = V'*V;
    aux2 = 1./sqrt(diag(aux1));
    aux3 = repmat(aux2',nout,1);
    V = V.*aux3; % % now V contains unit vectors in columns
    R2(iter) = (V'*A*V)/(V'*T*V);
    if iter>1 && (R2(iter) < R2(iter-1)+epsi)
        R2(iter) = NaN;
        break
    end
    
    cv = (X-repmat(mean(X),N,1))*V;
    Mprev = M;

    if samevariance
        % assume same variances
        thresh = (mean(cv( M, :)) + mean(cv(~M, :)))/2;
        M = cv > thresh;
    else    
        % assume different variances
        m1 = mean(cv( M, :));
        s1 =  var(cv( M, :));
        m2 = mean(cv(~M, :));
        s2 =  var(cv(~M, :));
        a = (1/s1-1/s2);
        b = 2*(m2/s2-m1/s1);
        c = (m1^2/s1-m2^2/s2)+ log(s1/s2);
        dis = sqrt(b^2 - 4*a*c);
        root1 = (-b + dis)/2/a;
        root2 = (-b - dis)/2/a;
        if (m1<root1 && root1<m2) || (m2<root1 && root1<m1) %m1<root1<m2 || m2<root1<m1
            thresh = root1;
        else
            thresh = root2;
        end
        M = cv > thresh;
    end
% sign of eigen vector arbitrary, foreground = true
    if size(find(Mprev & M), 1) < size(find(Mprev & ~M), 1)
        M = ~M;
        cv = -cv;
%         disp('negate cv and mask')
    end
    
    figure
    subplot(121), imshow(reshape(Mprev,nrows,ncols),[])
    subplot(122), imshow(reshape(cv,nrows,ncols),[])
    figure
    subplot(211), [yhi xhi] = hist(cv(:), 100); bar(xhi, yhi);
    subplot(212), plot(R2,'-o'),xlim([0 itermax]),ylim([0 1]);
end

cv = reshape(cv,nrows,ncols,nout);
M  = reshape(M ,nrows,ncols);
